# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The core work of Achille.
"""

import achille.indexes as indexes
import achille.transforms as transforms
import achille.lib.log as log

from achille.lib import files
from achille.config.constants import LOG_WORKER


class GenerationOptions(dict):
    """A specialized dictionnary.
    """

    OPTION_INDEXES = 'indexes'
    OPTION_DOCBOOK = 'docbook'
    OPTION_FILES = 'files'
    OPTION_OVERWRITE = 'overwrite'
    OPTION_VALIDATE = 'validate'
    OPTION_ASK = 'ask'

    def __init__(self):
        dict.__init__(self, ((self.OPTION_INDEXES, True),
                             (self.OPTION_DOCBOOK, True),
                             (self.OPTION_FILES, True),
                             (self.OPTION_OVERWRITE, True),
                             (self.OPTION_VALIDATE, False),
                             (self.OPTION_ASK, False)))


    def __str__(self):
        lLine = 'Options: '
        for lKey in self:
            if lKey != self.OPTION_ASK and self[lKey] == True:
                lLine = lLine + str(lKey) + ' '
        return lLine


        
class Worker(object):
    def generate(cls, aDescriptor, aOptions):
        log.get(LOG_WORKER).info(str(aOptions))
        log.get(LOG_WORKER).info(('Processing on %s' % aDescriptor.title))


        try:
            lValidate = aOptions[GenerationOptions.OPTION_VALIDATE]
        
            for lSection in aDescriptor.sections:
                log.get(LOG_WORKER).info(('Working on %s(%s)'
                                          % (lSection.title, lSection.id) ))

                # Build indexes
                if aOptions[GenerationOptions.OPTION_INDEXES] == True:
                    if lSection.index != None:
                        indexes.getIndexBuilderFor(lSection).run()

                # Generate documents
                if aOptions[GenerationOptions.OPTION_DOCBOOK] == True:
                    transforms.processTransformationsOn(
                        aDescriptor.target, lSection, lValidate)
                    
                # Copy files
                if aOptions[GenerationOptions.OPTION_FILES] == True:
                    if len(lSection.copiedFiles):
                        files.FileSystemInterface().copy(
                            aDescriptor.target, lSection)

        except Exception, e:
            log.get(LOG_WORKER).error(e)
        else:
            log.get(LOG_WORKER).info('Done')
    generate = classmethod(generate)


    def check(cls, aDescriptor):
        # [TODO] we should check too that source files given
        # in descriptor exist
        log.get(LOG_WORKER).info('Done')

    check = classmethod(check)


    def clean(cls, aDescriptor):
        # Delete target
        files.removePath(aDescriptor.target)
    
    clean = classmethod(clean)

