# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gtk.glade
import gobject

from achille.config import Descriptor, Section, CssSet
from achille.config.constants import *
from achille.gui.gtk import GLADE_FILE

class Tab(gtk.ScrolledWindow):
    def __init__(self):
        gtk.ScrolledWindow.__init__(self)


    def getId(self):
        raise NotImplementedError()

    

class FileTab(Tab):
    def __init__(self, aLink, aBuffer):
        Tab.__init__(self)

        self._name = aLink
        
        lView = gtk.TextView(aBuffer)
        lView.show()

        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        self.add(lView)
        self.show()


    def getId(self):
        return (self._name, self)



class DescriptorTab(gtk.ScrolledWindow):
    TITLE_ENTRY = 'titleEntry'
    TARGET_ENTRY = 'targetEntry'
    INDEX_ENTRY = 'indexEntry'
    ARTICLE_ENTRY = 'articleEntry'
    LISTING_ENTRY = 'filesEntry'
    CSS_VIEW = 'cssTreeview'

    class ListController(object):
        COLUMN_NAME = 0
        COLUMN_FILE = 1
        COLUMN_PRINCIPAL = 2
        COLUMN_TRUE = 3
        
        def __init__(self, aView):
            self._cssSet = CssSet()
            self._cssList = self._cssSet[CSS_TYPE_ALTERNATE]
            
            self._view = aView
            self._store = gtk.ListStore(
                gobject.TYPE_STRING,
                gobject.TYPE_STRING,
                gobject.TYPE_BOOLEAN,
                gobject.TYPE_BOOLEAN
                )
            self._view.set_model(self._store)
                        
            lCell = gtk.CellRendererText()
            lCell.connect('edited', self.onEdition, self.COLUMN_NAME) 
            lColumn = gtk.TreeViewColumn('Name', lCell)
            lColumn.add_attribute(lCell, 'text', self.COLUMN_NAME)
            lColumn.add_attribute(lCell, 'editable', self.COLUMN_TRUE)
            self._view.append_column(lColumn)

            lCell = gtk.CellRendererText()
            lCell.connect('edited', self.onEdition, self.COLUMN_FILE)
            lColumn = gtk.TreeViewColumn('Filename', lCell)
            lColumn.add_attribute(lCell, 'text', self.COLUMN_FILE)
            lColumn.add_attribute(lCell, 'editable', self.COLUMN_TRUE)
            self._view.append_column(lColumn)

            lCell = gtk.CellRendererToggle()
            lCell.connect('toggled', self.onOrdering, self.COLUMN_PRINCIPAL)
            lColumn = gtk.TreeViewColumn('Primary', lCell)
            lColumn.add_attribute(lCell, 'radio', self.COLUMN_TRUE)
            lColumn.add_attribute(lCell, 'activatable', self.COLUMN_TRUE)
            self._view.append_column(lColumn)

        
        def buildFrom(self, aCssSet):
            self._cssSet = aCssSet
            self._cssList = self._cssSet[CSS_TYPE_ALTERNATE]

            self._store.clear()
            try:
                lName, lFilename = aCssSet[CSS_TYPE_NORMAL]
                self._store.append([lName, lFilename, True, True])
            except TypeError:
                pass

            try:
                for lTuple in aCssSet[CSS_TYPE_ALTERNATE]:
                    self._store.append([lTuple[0], lTuple[1], False, True])
            except TypeError:
                pass


        def onEdition(self, aCell, aRow, aNewValue, aColumn):
            lIter = self._store.get_iter_from_string(aRow)

            # Update model
            lSheetName = self._store.get_value(lIter, self.COLUMN_NAME)
            lSheetFile = self._store.get_value(lIter, self.COLUMN_FILE)
            
            if self._store.get_value(lIter, self.COLUMN_PRINCIPAL):
                if aColumn == self.COLUMN_NAME:
                    self._cssSet[CSS_TYPE_NORMAL] = aNewValue, lSheetFile
                elif aColumn == self.COLUMN_FILE:
                    self._cssSet[CSS_TYPE_NORMAL] = lSheetName, aNewValue

            else:
                self._cssList.remove((lSheetName, lSheetFile))
                if aColumn == self.COLUMN_NAME:
                    self._cssList.append((aNewValue, lSheetFile))
                elif aColumn == self.COLUMN_FILE:
                    self._cssList.append((lSheetName, aNewValue))
                
            # Update list
            if aColumn == self.COLUMN_NAME or aColumn == self.COLUMN_FILE:
                self._store.set(lIter, aColumn, aNewValue)


        def onOrdering(self, aCell, aRow, aColumn):
            lIter = self._store.get_iter_from_string(aRow)

            # Update model
            lSheetName = self._store.get_value(lIter, self.COLUMN_NAME)
            lSheetFile = self._store.get_value(lIter, self.COLUMN_FILE)

            self._cssSet.makeSheetPrincipal(lSheetName)

            # Update list
            for lRow in self._store:
                # TODO
                print lRow, aColumn
                lRow[aColumn] = False
            self._store[aRow][aColumn] = True


        def onAddCssClicked(self, aWidget):
            lNewFile = 'stylesheet.css'
            
            # Update model
            self._cssSet[CSS_TYPE_ALTERNATE].append(('', lNewFile))
            
            # Update list
            self._store.append(['', lNewFile, False, True])


        def onRemoveCssClicked(self, aWidget):
            lSelection = self._view.get_selection()
            lSelected = lSelection.get_selected()
            if lSelected:
                try:
                    lStore, lIter = lSelected

                    # Update model
                    lSheetName = self._store.get_value(lIter, self.COLUMN_NAME)
                    lSheetFile = self._store.get_value(lIter, self.COLUMN_FILE)

                    if self._store.get_value(lIter, self.COLUMN_PRINCIPAL):
                        self._cssSet[CSS_TYPE_NORMAL] = None
                        
                    else:
                        self._cssList.remove((lSheetName, lSheetFile))
            
                    # Update list
                    lStore.remove(lIter)
                except TypeError:
                    pass
        

    def __init__(self, aGladeFile, aDescriptor):
        gtk.ScrolledWindow.__init__(self)
        
        self._descriptor = aDescriptor

        self._design = gtk.glade.XML(aGladeFile, 'descriptorWindow')
        self._cssView = DescriptorTab.ListController(
            self._design.get_widget(self.CSS_VIEW))
        self._initialize()
        lDict = {
            'on_titleEntry_changed': (self.onTitleChanged),
            'on_targetEntry_changed': (self.onTargetChanged),
            'on_indexEntry_changed': (self.onIndexChanged),
            'on_articleEntry_changed': (self.onArticleChanged),
            'on_listingEntry_changed': (self.onListingChanged),
            'on_addCss_clicked': (self._cssView.onAddCssClicked),
            'on_removeCss_clicked': (self._cssView.onRemoveCssClicked)
            }
        self._design.signal_autoconnect(lDict)
        
        lBox = self._design.get_widget('vbox')
        lBox.unparent()

        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.add_with_viewport(lBox)
        self.show()


    def _initialize(self):
        self._design.get_widget(self.TITLE_ENTRY).set_text(
            self._descriptor.title)
        self._design.get_widget(self.TARGET_ENTRY).set_text(
            self._descriptor.target)
        self._design.get_widget(self.INDEX_ENTRY).set_text(
            self._descriptor.xsltSet[XSLT_TYPE_INDEX])
        self._design.get_widget(self.ARTICLE_ENTRY).set_text(
            self._descriptor.xsltSet[XSLT_TYPE_ARTICLE])
        self._design.get_widget(self.LISTING_ENTRY).set_text(
            self._descriptor.xsltSet[XSLT_TYPE_FILES])
        self._cssView.buildFrom(self._descriptor.cssSet)
        

    def getId(self):
        return (self._descriptor, self)


    def onTitleChanged(self, aEntry):
        #print 'Title changed'
        self._descriptor.setTitle(aEntry.get_text())


    def onTargetChanged(self, aEntry):
        #print 'target changed'
        self._descriptor.setTarget(aEntry.get_text())
    

    def onIndexChanged(self, aEntry):
        #print 'index changed'
        self._descriptor.xsltSet[XSLT_TYPE_INDEX] = aEntry.get_text()


    def onArticleChanged(self, aEntry):
        #print 'article changed'
        self._descriptor.xsltSet[XSLT_TYPE_ARTICLE] = aEntry.get_text()


    def onListingChanged(self, aEntry):
        #print 'listing changed'
        self._descriptor.xsltSet[XSLT_TYPE_FILES] = aEntry.get_text()



class SectionTab(gtk.ScrolledWindow):
    TITLE_ENTRY = 'titleEntry'
    SOURCE_ENTRY = 'sourceEntry'
    TARGET_ENTRY = 'targetEntry'

    TYPE_COMBO = 'typeCombo'
    TYPE_ENTRY = 'typeComboEntry'
    SORT_COMBO = 'sortCombo'
    SORT_ENTRY = 'sortComboEntry'
    MAX_BUTTON = 'maxSpinbutton'
    FEATURED_ENTRY = 'featuredEntry'
    SECTIONS_ENTRY = 'sectionsEntry'
    IGNORED_ENTRY = 'ignoredEntry'
    
    DOCBOOK_BUTTON = 'docbookButton'
    FILES_ENTRY = 'filesEntry'
    

    def __init__(self, aGladeFile, aSection):
        gtk.ScrolledWindow.__init__(self)
        
        self._section = aSection

        self._design = gtk.glade.XML(aGladeFile, 'sectionWindow')
        self._initialize()
        lDict = {
            'on_titleEntry_changed': (self.onTitleChanged),
            'on_sourceEntry_changed': (self.onSourceChanged),
            'on_targetEntry_changed': (self.onTargetChanged),
            'on_typeComboList_selection_changed': (self.onTypeChanged),
            'on_sortComboList_selection_changed': (self.onSortChanged),
            'on_maxSpinbutton_changed': (self.onMaxChanged),
            'on_featuredEntry_changed': (self.onFeaturedChanged),
            'on_sectionsEntry_changed': (self.onSectionsChanged),
            'on_ignoredEntry_changed': (self.onIgnoredChanged),
            'on_docbookButton_toggled': (self.onDocbookToggled),
            'on_filesEntry_changed': (self.onFilesChanged)
            }
        self._design.signal_autoconnect(lDict)

        lBox = self._design.get_widget('vbox')
        lBox.unparent()

        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.add_with_viewport(lBox)
        self.show()


    def _initialize(self):
        self._design.get_widget(self.TITLE_ENTRY).set_text(
            self._section.title)
        self._design.get_widget(self.SOURCE_ENTRY).set_text(
            str(self._section.source))
        self._design.get_widget(self.TARGET_ENTRY).set_text(
            self._section.target)

        if self._section.index != None:
            self._initializeIndex()
            
        lButton = self._design.get_widget(self.DOCBOOK_BUTTON)
        if not self._section.transformedFiles:
            lButton.set_label('No')
            lButton.set_active(False)

        lFilesList = ''
        for lFile in self._section.copiedFiles:
            lFilesList = lFilesList + lFile + ', '
        self._design.get_widget(self.FILES_ENTRY).set_text(
            lFilesList)


    def _initializeIndex(self):
        self._design.get_widget(self.TYPE_ENTRY).set_text(
            self._section.index.type)

        self._design.get_widget(self.SORT_COMBO).set_sensitive(gtk.TRUE)
        self._design.get_widget(self.SORT_ENTRY).set_text(
            self._section.index.sort)
        

        self._design.get_widget(self.MAX_BUTTON).set_sensitive(gtk.TRUE)
        self._design.get_widget(self.MAX_BUTTON).set_value(
            self._section.index.max)

        self._design.get_widget(self.FEATURED_ENTRY).set_sensitive(gtk.TRUE)
        self._design.get_widget(self.FEATURED_ENTRY).set_text(
            self._section.index.featuredFile)

        lSectionsList = ''
        for lSection in self._section.index.sectionIds:
            lSectionsList = lSectionsList + lSection + ', '
        self._design.get_widget(self.SECTIONS_ENTRY).set_sensitive(gtk.TRUE)
        self._design.get_widget(self.SECTIONS_ENTRY).set_text(
            lSectionsList)

        lIgnoredList = ''
        for lFile in self._section.index.ignoredFiles:
            lIgnoredList = lIgnoredList + lFile + ', '
        self._design.get_widget(self.IGNORED_ENTRY).set_sensitive(gtk.TRUE)
        self._design.get_widget(self.IGNORED_ENTRY).set_text(
            lIgnoredList)


    def getId(self):
        return (self._section, self)


    def onTitleChanged(self, aEntry):
        print 'Title changed'


    def onSourceChanged(self, aEntry):
        print 'source changed'


    def onTargetChanged(self, aEntry):
        print 'target'

        
    def onTypeChanged(self, aList):
        print 'menu changed'
        try:
            lType = aList.get_selection()[0].child.get_text()
            self._section.index.setType(lType)
            if lType == INDEX_TYPE_NONE:
                self._design.get_widget(self.SORT_COMBO).set_sensitive(gtk.FALSE)
                self._design.get_widget(self.MAX_BUTTON).set_sensitive(gtk.FALSE)
                self._design.get_widget(self.FEATURED_ENTRY).set_sensitive(gtk.FALSE)
                self._design.get_widget(self.SECTIONS_ENTRY).set_sensitive(gtk.FALSE)
                self._design.get_widget(self.IGNORED_ENTRY).set_sensitive(gtk.FALSE)
            else:
                self._design.get_widget(self.SORT_COMBO).set_sensitive(gtk.TRUE)
                self._design.get_widget(self.MAX_BUTTON).set_sensitive(gtk.TRUE)
                self._design.get_widget(self.FEATURED_ENTRY).set_sensitive(gtk.TRUE)
                self._design.get_widget(self.SECTIONS_ENTRY).set_sensitive(gtk.TRUE)
                self._design.get_widget(self.IGNORED_ENTRY).set_sensitive(gtk.TRUE)
        except IndexError:
            pass
        

    def onSortChanged(self, aList):
        if self._section.index != None:
            try:
                self._section.index.setSort(
                    aList.get_selection()[0].child.get_text())
            except IndexError:
                pass


    def onMaxChanged(self, aButton):
        if self._section.index != None:
            self._section.index.setMax(aButton.get_value_as_int())
        
    
    def onFeaturedChanged(self, aEntry):
        print 'TODO: featured changed'


    def onSectionsChanged(self, aEntry):
        print 'TODO: sections changed'


    def onIgnoredChanged(self, aEntry):
        print 'TODO: ignored changed'

        
    def onDocbookToggled(self, aButton):
        if aButton.get_active():
            aButton.set_label('Yes')
        else:
            aButton.set_label('No')
        self._section.transformedFiles = aButton.get_active()


    def onFilesChanged(self, aEntry):
        print 'TODO: files changed'



class NotebookManager(object):
    def __init__(self, aNotebook):
        self._book = aNotebook
        self._tabs = []

        
    def open(self, aLink):
        """Open a file, a description or a section for editing.

        If the type of the aLink parameter is a simple string, it must contain
        a valid filename. A textView widget is opened so as to edit the file.
        If this parameter is a section or a descriptor, a special tab is
        opened.

        Each file or section is only opened once. If a file or a section is to
        be reopened, its tab becomes the current tab.
        """
        lWidget = None
        for lId, lWidget in self._tabs:
            if lId == aLink:
                self._book.set_current_page(self._book.page_num(lWidget))
                return
            
        if type(aLink) is unicode or type(aLink) is str:
            lWidget = self._createFileTab(aLink)
        elif type(aLink) is Section:
            lWidget = self._createSectionTab(aLink)
        elif type(aLink) is Descriptor:
            lWidget = self._createDescriptorTab(aLink)
            
        if lWidget != None:
            self._tabs.append((aLink, lWidget))
            self._book.set_current_page(self._book.page_num(lWidget))
        else:
            raise AttributeError('Invalid Link: %s' % (aLink))

    def closeCurrentTab(self):
        lPage = self._book.get_current_page()
        if lPage != -1:
            lChild = self._book.get_nth_page(lPage)
            for lId, lWidget in self._tabs:
                if lWidget == lChild:
                    self._tabs.remove((lId, lWidget))
                    continue
            self._book.remove_page(lPage)
                

    def _createFileTab(self, aLink):
        # Open file
        try:
            lBuffer = self._createBuffer(aLink)
        except IOError, lMessage:
            print lMessage
            lDialog = gtk.Dialog("Error",
                                 None,
                                 gtk.DIALOG_MODAL,
                                 (gtk.STOCK_CLOSE, gtk.RESPONSE_OK))
            lLabel = gtk.Label(str(lMessage))
            lLabel.show()
            lDialog.vbox.pack_start(lLabel, gtk.TRUE, gtk.TRUE, 10)
            lDialog.connect('response',
                            (lambda aDialog, aArg: aDialog.hide()))
            lDialog.show()
            return None

        # Create Label
        lLabel = gtk.Label(aLink)
        lLabel.show()

        # Create Tab
        lTab = FileTab(aLink, lBuffer)
        
        # Register it
        self._book.append_page(lTab, lLabel)
        
        return lTab



    def _createSectionTab(self, aSection):
        # Create Label
        lLabel = gtk.Label(aSection.id)
        lLabel.show()

        # Create Tab
        lTab = SectionTab(GLADE_FILE, aSection)
        
        # Register it
        self._book.append_page(lTab, lLabel)
        
        return lTab


    def _createDescriptorTab(self, aDescriptor):
        # Create Label
        lLabel = gtk.Label(aDescriptor.filename)
        lLabel.show()

        # Create Tab
        lTab = DescriptorTab(GLADE_FILE, aDescriptor)

        # Register it
        self._book.append_page(lTab, lLabel)

        return lTab
        
        
    def _createBuffer(self, aLink):
        lFile = open(aLink, 'r')
        lBuffer = gtk.TextBuffer()
        for lLine in lFile:
            lBuffer.insert_at_cursor(lLine.decode('latin-1', 'replace'))
        lFile.close()
        return lBuffer
