# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import gtk

from achille.gui.gtk import GLADE_FILE
        
class TreeController(object):
    def __init__(self, aApp, aView):
        self._application = aApp
        self._view = aView
        self._store = gtk.TreeStore(str)
        self._view.set_model(self._store)

        # Last event doesn't store the event object but a tuple containing
        # the x and y values, because the event x and y values evolve when
        # the current window changes
        self._lastEvent = None
        
        lCell = gtk.CellRendererText()
        lColumn = gtk.TreeViewColumn('Column 0', lCell)
        lColumn.add_attribute(lCell, 'text', 0)

        self._view.append_column(lColumn)

        # Plug the Descriptor popup menu
        lDesign = gtk.glade.XML(GLADE_FILE, 'descriptorMenu')
        self._descriptorMenu = lDesign.get_widget('descriptorMenu')
        lDict = {
            'on_new_activate': (self.onMenuNewSection),
            'on_save_activate': (self._application.onMenuSave),
            'on_saveas_activate': (self._application.onMenuSaveAs),
            'on_generate_activate': (self._application.onMenuGenerate, False),
            'on_properties_activate': (self.onMenuProperties)
            }
        lDesign.signal_autoconnect(lDict)

        # Plug the Sections popup menu
        lDesign = gtk.glade.XML(GLADE_FILE, 'sectionMenu')
        self._sectionMenu = lDesign.get_widget('sectionMenu')
        lDict = {
            'on_newSection_activate': (self.onMenuNewSection),
            'on_duplicate_activate': (self.onMenuSectionDuplicate),
            'on_properties_activate': (self.onMenuSectionProperties)
            }
        lDesign.signal_autoconnect(lDict)

        
    def buildFrom(self, aConfiguration):
        self._store.clear()
        lRootIter = self._store.append(None, [aConfiguration.title])
        for lSection in aConfiguration.sections:
            self._store.append(lRootIter, [lSection.id])
        self._view.expand_all()


    def onSelection(self, aView, aPath, aColumn):
        if len(aPath) == 2:
            self._application.openSectionFor(
                self._store.get_value(self._store.get_iter(aPath), 0))
        else:
            self._application.openDescriptor()
            

    def onButtonPressEvent(self, aWidget, aEvent):
        if self._store.get_iter_first() == None or aEvent.button != 3:
            return

        self._lastEvent = aEvent.get_coords()
        lPath = self._view.get_path_at_pos(int(aEvent.x), int(aEvent.y))[0]
        if len(lPath) == 2:
            self._sectionMenu.popup(
                None, None, None, aEvent.button, aEvent.time)
        else:
            self._descriptorMenu.popup(
                None, None, None, aEvent.button, aEvent.time)
        

    def onMenuNewSection(self, aWidget):
        lSection = self._application.getDescriptor().addSection()
        self._application.openSectionFor(lSection.id)
        self._store.append(self._store.get_iter_root(), [lSection.id])
        self._view.expand_all()
        

    def onMenuProperties(self, aWidget):
        self._application.openDescriptor()
        

    def onMenuSectionDuplicate(self, aWidget):
        print 'Section Duplicate...'


    def onMenuSectionProperties(self, aWidget):
        lPath = self._view.get_path_at_pos(
            int(self._lastEvent[0]), int(self._lastEvent[1]))[0]
        self._application.openSectionFor(
            self._store.get_value(self._store.get_iter(lPath), 0))

        
