# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os

from Ft.Lib import Uri
from Ft.Xml import XPath
from Ft.Xml import Domlette
from Ft.Xml import EMPTY_NAMESPACE

import achille.lib.log as log

# FIXME
from indexbuilder import *
from achille.lib import files
from achille.config import site
from achille.config.constants import LOG_WORKER

ARTICLE_TITLE_XPATH    = '/article/title'
ARTICLE_DATE_XPATH = '/article/artheader/date'
ARTICLE_ABSTRACT_XPATH = '/article/artheader/abstract'

DTD_PUBLIC_ID = u'-//ACHILLE//DTD Achille-files XML V4.2//EN'
DTD_SYSTEM_ID = os.path.join(
    site.CONFIGURATION_PATH, u'dtd', u'achille-file-index.dtd')

class FilesIndexBuilder(IndexBuilder):
    def __init__(self, aSection):
        IndexBuilder.__init__(self, aSection)
        
    def run(self):
        log.get(LOG_WORKER).info(
            ('\tBuilding files index in %s' % self._section.source))
        for lDir in self._section.index.dirs:
            self.readDirectory(lDir)
        
        self.createIndex(DTD_PUBLIC_ID, DTD_SYSTEM_ID, EMPTY_NAMESPACE, 'index')
        lListingNode = self._document.createElementNS(EMPTY_NAMESPACE, 'listing')
        self._document.documentElement.appendChild(lListingNode)
        
        # The files map
        lFiles = {}
        for lFileList in self._files.values():
            for lFile in lFileList:
                if not lFile in self._section.index.ignoredFiles:
                    lKey, lNode = self.createFileNode(lFile)
                    if (lFiles.has_key(lKey)):
                        log.get(aLOG_WORKER).warn(
                            ('\t Duplicate key: %s' % lKey))
                    lFiles[lKey] = lNode

        lKeys = lFiles.keys()
        lKeys.sort()
        if self._section.index.sort in [INDEX_SORT_DATE, '']:
            lKeys.reverse()
        for lKey in lKeys:
            lListingNode.appendChild(lFiles[lKey])

        self.writeIndex(files.FileSystemInterface.TYPE_DTD_FILES_INDEX)
        

    def createFileNode(self, aFilename):
        log.get(LOG_WORKER).debug(('\tReading file: %s' % aFilename))

        lSize, lDate = files.getFileInfo(aFilename)
        lShortName = os.path.basename(aFilename)
        lNewNode = self._document.createElementNS(EMPTY_NAMESPACE, 'file')
        lNewNode.setAttributeNS(EMPTY_NAMESPACE, 'filename', lShortName)
        lNewNode.setAttributeNS(EMPTY_NAMESPACE, 'size', lSize)
        lNewNode.setAttributeNS(EMPTY_NAMESPACE, 'date', lDate)
        lNewNode.setAttributeNS(EMPTY_NAMESPACE, 'description', '')

        # Choose the key to return
        #         lSplitDate = lDate.split('/')
        #         lKey = lSplitDate[2] + lSplitDate[1] + lSplitDate[0]
        #         lKey += str(self._nbArticles)
        #         if self._section.index.sort == INDEX_SORT_FILE:
        #             lKey = aFilename
        lKey = aFilename
        
        return lKey, lNewNode
