# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import logging, logging.handlers, sys



DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL


def __configure(aName, aLevel, aHandler):
    lLogger = logging.getLogger(aName)
    lFormatter = logging.Formatter(
        '%(module)-10s:%(lineno) 4d\t%(levelname)s\t%(message)s')
    aHandler.setFormatter(lFormatter)
    lLogger.addHandler(aHandler)
    lLogger.setLevel(aLevel)
    return lLogger

        
def createWithFilename(aName = 'default',
                       aLevel = INFO,
                       aFilename = 'log'):
    return __configure(aName, aLevel, logging.FileHandler(aFilename))


def createWithStream(aName = 'default',
                     aLevel = INFO,
                     aStream = sys.stdout):
    return __configure(aName, aLevel, logging.StreamHandler(aStream))


def createWithSocket(aName = 'default',
                     aLevel = INFO,
                     aHost = 'localhost',
                     aPort = 5000):
    return __configure(
        aName, aLevel, logging.handlers.SocketHandler(aHost, aPort))

        
def get(aName = None):
    return logging.getLogger(aName)
    
