# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, re

from Ft.Lib import Uri, UriException
from Ft.Xml.InputSource import DefaultFactory
from Ft.Xml.Domlette import NonvalidatingReader, ValidatingReader, NoExtDtdReader
from Ft.Xml.Xslt import Processor

import achille.lib.log as log

# FIXME
from achille.config.constants import *

from achille.config import site
from achille.lib import files, date


def processXSLVariables(aSection):
    """Generate variables for XSLT transformation.

    This method builds the variable.xsl file, from the variable.xsl.in
    pattern. The generated file holds textual information (like a title
    or Achille version) and some information useful for articles
    transformation (the path to the site root). Since some information
    is section dependant (like paths), this file is regenerated for each
    section built in the site. 
    """
    
    log.get(LOG_TRANSFORM).debug('\tBuilding XSLT variables')

    # Load pattern file.
    lPatternFile = open(
        os.path.join(site.CONFIGURATION_PATH, 'xsl', 'variables.xsl.in'), 'r')
    lPattern = lPatternFile.read().encode('latin-1')
    lPatternFile.close()

    lTitle = aSection.descriptor.title + ': ' + aSection.title
    lCssSet = aSection.descriptor.cssSet
    lRootPath = files.getPathToRoot(aSection.target[1:])

    # Prepare Menu tags
    lMenu = ''
    for lSection in aSection.descriptor.sections:
        if lSection.index != None:
            lMenu += '<a href="' + os.path.join(lRootPath,
                                                lSection.target[1:])
            lMenu += '">' + lSection.title + u'</a>\n  '

    # Prepare CSS tags
    lCssName, lCssSheet = lCssSet[CSS_TYPE_NORMAL]
    lCssSheets = '<link rel="stylesheet" type="text/css"'
    lCssSheets += ' href="' + os.path.join(lRootPath, lCssSheet[1:])
    lCssSheets += '" title="' + lCssName + '" />\n'
    
    for lCssName, lCssSheet in lCssSet[CSS_TYPE_ALTERNATE]:
        lCssSheets += '<link rel="alternate stylesheet" type="text/css"'
        lCssSheets += ' href="' + os.path.join(lRootPath, lCssSheet[1:]) + '"'
        lCssSheets += ' title="' + lCssName + '" />\n'

    # Writing variables.xsl file
    lResult = lPattern
    lResult = re.sub('@TITLE@', lTitle, lResult)
    lResult = re.sub('@MENU@', lMenu, lResult)
    lResult = re.sub('@STYLESHEETS@', lCssSheets, lResult)
    lResult = re.sub('@ACHILLE_VERSION@', site.VERSION, lResult)
    lResult = re.sub('@PATH_TO_ROOT@', lRootPath, lResult)

    lOutputFile = open(os.path.join(
        site.CONFIGURATION_PATH, 'xsl', 'variables.xsl'), 'w')
    lOutputFile.write(lResult.encode('latin-1'))
    lOutputFile.close()


def processXSLTransformationsOn(aWorkingDir, aSection, aValidate):
    # First make sure the file can be created
    files.buildPathFor(aWorkingDir, aSection)
    lDirectoryMap = files.FileSystemInterface.get().readDirectory(
        aSection.source())
    # log.get(LOG_TRANSFORM).debug(('\tFiles: %s' % str(lDirectoryMap)))

    try:
        log.get(LOG_TRANSFORM).debug(('\tPreparing transformations for %s' %
                            files.FileSystemInterface.TYPE_SCHEMA_RDF))
        lFiles = lDirectoryMap[files.FileSystemInterface.TYPE_SCHEMA_RDF]
	for lArticle in lFiles:
            doTransform(aWorkingDir, aSection, lArticle,
                        files.FileSystemInterface.TYPE_SCHEMA_RDF, aValidate)
    except KeyError, e:
        log.get(LOG_TRANSFORM).debug(('\tNo key: %s' % e))

    try:
        if aSection.transformedFiles != 0:
            log.get(LOG_TRANSFORM).debug(
                ('\tPreparing transformations for %s' %
                 files.FileSystemInterface.TYPE_DTD_DOCBOOK))
            lFiles = lDirectoryMap[files.FileSystemInterface.TYPE_DTD_DOCBOOK]
            for lArticle in lFiles:
                doTransform(aWorkingDir, aSection, lArticle,
                            files.FileSystemInterface.TYPE_DTD_DOCBOOK, aValidate)
    except KeyError, e:
        log.get(LOG_TRANSFORM).debug(('\tNo key: %s' % e))

    try:
        log.get(LOG_TRANSFORM).debug(('\tPreparing transformations for %s' %
                            files.FileSystemInterface.TYPE_DTD_FILES_INDEX))
        lFiles = lDirectoryMap[files.FileSystemInterface.TYPE_DTD_FILES_INDEX]
        for lFile in lFiles:
            doTransform(aWorkingDir, aSection, lFile,
                        files.FileSystemInterface.TYPE_DTD_FILES_INDEX, aValidate)
    except KeyError, e:
        log.get(LOG_TRANSFORM).debug(('\tNo key:  %s' % e))


#
# TODO: simplify and group these methods by joining constant definitions
#
def doTransform(aWorkingDir, aSection, aFile, aType, aValidate):
    lSheetFile = ''
    lPath, lSrcFile = os.path.split(aFile)
    lTargetFile = os.path.splitext(lSrcFile)[0]
    lTargetFile = os.path.join(
        aWorkingDir, aSection.target[1:], lTargetFile) + '.html'

    if aType == files.FileSystemInterface.TYPE_DTD_DOCBOOK:
        lSheetFile = os.path.join(
            site.CONFIGURATION_PATH,
            aSection.descriptor.xsltSet[XSLT_TYPE_ARTICLE])

    elif aType == files.FileSystemInterface.TYPE_SCHEMA_RDF:
        lSheetFile = os.path.join(
            site.CONFIGURATION_PATH,
            aSection.descriptor.xsltSet[XSLT_TYPE_INDEX])
            
    elif aType == files.FileSystemInterface.TYPE_DTD_FILES_INDEX:
        lSheetFile = os.path.join(
            site.CONFIGURATION_PATH,
            aSection.descriptor.xsltSet[XSLT_TYPE_FILES])
    
    doTransformNow(aFile, lTargetFile, lSheetFile, aValidate)

def doTransformNow(aFile, aTargetFile, aSheetFile, aValidate):
    log.get(LOG_TRANSFORM).info(('\tApplying transformations on %s' % aFile))

    lReader = NoExtDtdReader
    if aValidate:
        lReader = ValidatingReader
    lProc = Processor.Processor(documentReader = lReader)
    
    # Load Sheets and Document. Sheet URIs must be absolute !
    try:
        lSheet = DefaultFactory.fromUri(
            Uri.OsPathToUri(os.path.abspath(aSheetFile)))
        lDoc = DefaultFactory.fromUri(Uri.OsPathToUri(aFile))
        
        lProc.appendStylesheet(lSheet)
        
        # Go...
        lProc.run(iSrc = lDoc,
                  topLevelParams = {'generated_date': date.getFullDate()},
                  outputStream = open(aTargetFile, 'w'))
    except UriException, e:
        log.get(LOG_TRANSFORM).error(('\t** %s' % e))
