#!/usr/bin/env python

# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, stat, sys, time

from distutils.core import setup
from distutils.util import subst_vars
from distutils.command.install import install
from distutils import log


VERSION = '2.0'
HEADER = """
# Achille. An assistant to build static websites compliant to W3C standards.
# Copyright (C) 2003 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of Achille.
#
# Achille is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Generated at install time

"""


# Redifine build_py of distutils.command.build_py so as to
# generate our configuration module on the fly.
class AchilleInstaller(install):
    """Subclass of install from distutils.
    
    [FIXME]
    """
    def run (self):
        # Obviously have to build before we can install
        if not self.skip_build:
            self.run_command('build')
            
        self.generate_conf()

        if not self.dry_run:
            self.finish_run()
        

    def finish_run(self):
        #
        # Mainly ripped from distutils.command.install.install.py
        #
        # Run all sub-commands (at least those that need to be run)
        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)

        # Add sufficient rights to create contextual stylesheet
        path_to_xsl_sheets = os.path.sep.join(
            [self.install_data, 'share', 'achille', 'xsl'])
        log.info('giving access to : ' + path_to_xsl_sheets)
        os.chmod(path_to_xsl_sheets,
                 stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR |
                 stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP |
                 stat.S_IROTH | stat.S_IWOTH | stat.S_IXOTH)
        os.chmod(os.path.join(path_to_xsl_sheets, 'variables.xsl.in'),
                 stat.S_IRUSR | stat.S_IWUSR | 
                 stat.S_IRGRP | stat.S_IWGRP | 
                 stat.S_IROTH | stat.S_IWOTH )
 

        if self.path_file:
            self.create_path_file()

        # write list of installed files, if requested.
        if self.record:
            outputs = self.get_outputs()
            if self.root:
                root_len = len(self.root)
                for counter in xrange(len(outputs)):
                    outputs[counter] = outputs[counter][root_len:]
            self.execute(write_file,
                         (self.record, outputs),
                         "writing list of installed files to '%s'" %
                         self.record)

        sys_path = map(os.path.normpath, sys.path)
        sys_path = map(os.path.normcase, sys_path)
        install_lib = os.path.normcase(os.path.normpath(self.install_lib))
        if (self.warn_dir
            and not (self.path_file and self.install_path_file)
            and install_lib not in sys_path):
            log.debug(("modules installed to '%s', which is not in "
                       "Python's module search path (sys.path) -- " 
                       "you'll have to change the search path yourself"),
                      self.install_lib)


    def generate_conf(self):
        outfile = os.path.sep.join(
            ['build', 'lib', 'achille', 'config', 'site.py'])
        log.info('generating configuration: ' + outfile)
        if self.dry_run:
            return
        
        output = file(outfile, 'w')
        output.write(HEADER)
        output.write('VERSION = u\'' + VERSION + '\'\n')
        output.write('GENERATION = u\'' + time.asctime() + '\'\n')

        path_to_configuration = os.path.sep.join(
            [self.install_data, 'share', 'achille'])
        output.write('CONFIGURATION_PATH = u\'' + path_to_configuration + '\'\n')

        path_to_script = os.path.join(self.install_scripts, 'run_achille.py')
        output.write('PROGRAM_PATH = u\'' + path_to_script + '\'\n')

        path_to_libs = os.path.join(self.install_lib, 'achille')
        output.write('LIBS_PATH = u\'' + path_to_libs + '\'\n')
        
        output.close()





setup(
    cmdclass = {'install': AchilleInstaller },
    name = "Achille",
    version = VERSION,
    url='https://savannah.nongnu.org/projects/achille/',
    description  = "A Docbook and XSLT based static website management tool",
    license = 'GPL http://www.gnu.org/copyleft/gpl.html',
    author = 'David Soulayrol',
    author_email = 'dsoulayrol@free.fr',
    platforms = ['posix', 'win32', 'cygwin'],
    packages = ['achille',
                'achille.config',
                'achille.gui',
                'achille.gui.gtk',
                'achille.indexes',
                'achille.lib',
                'achille.transforms'],
    scripts = ['run_achille.py'],
    data_files = [('share/achille/dtd', ['dtd/achille-file-index.dtd']),
                  ('share/achille/xsl', ['xsl/variables.xsl.in']),
                  ('share/achille/xsl/docbook', ['xsl/docbook/docbook.xsl',
                                                 'xsl/docbook/images.xsl',
                                                 'xsl/docbook/inline.xsl',
                                                 'xsl/docbook/lists.xsl',
                                                 'xsl/docbook/page.xsl',
                                                 'xsl/docbook/tables.xsl']),
                  ('share/achille/xsl/indexes', ['xsl/indexes/index.xsl',
                                                 'xsl/indexes/files.xsl']),
                  ('share/achille/xsl/lib', ['xsl/lib/strings.xsl']),
                  ('share/achille/gui/gtk', ['gui/gtk/gtk-achille.glade',
                                             'gui/gtk/gtk-achille.gladep'])]
     )
