<?xml version="1.0" encoding="ISO-8859-15"?>
<xsl:stylesheet version="1.0" 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!--

Achille. An assistant to build static websites compliant to W3C standards.
Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>

This file is part of Achille.

Achille is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-->

<xsl:include href="../variables.xsl" />
<xsl:include href="../lib/strings.xsl" />
<xsl:include href="images.xsl" />
<xsl:include href="inline.xsl" />
<xsl:include href="lists.xsl" />
<xsl:include href="tables.xsl" />

<xsl:template match="artheader">
</xsl:template>

<xsl:template match="section">
	<xsl:param name="level" select="0" />
	<xsl:apply-templates>
		<xsl:with-param name="level" select="$level+1" />
	</xsl:apply-templates>
</xsl:template>
	
<xsl:template match="sectioninfo/date">
</xsl:template>

<xsl:template match="article/title">
</xsl:template>

<!-- TODO: Simplifier en title avec <choose> -->
<xsl:template match="section/title">
	<xsl:param name="level" select="0" />
	<xsl:element name="h{$level}">
		  
		<xsl:apply-templates />
	</xsl:element>
</xsl:template>

<xsl:template match="ulink">
	<xsl:if test="not(ancestor::address)">
	<a>
		<xsl:attribute name="href">
			<xsl:call-template name="translate-url">
			<xsl:with-param name="url" select="attribute::url"/>
			</xsl:call-template>
		</xsl:attribute>
		<xsl:if test="@type">
			<xsl:attribute name="target">_blank</xsl:attribute>
		</xsl:if>
		<!-- <xsl:value-of select="text()" /> -->
		<xsl:apply-templates />
	</a>
	</xsl:if>
</xsl:template>

<xsl:template match="answer">
	<div class="answer">
		<span class="answermark">Rponse :</span>
		<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="blockquote">
	<div class="blockquote"><xsl:apply-templates /></div>
</xsl:template>

<xsl:template match="caution">
	<div class="caution">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
        </xsl:if>
	<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="important">
	<div class="important">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
	</xsl:if>
	<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="note">
	<div class="note">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
	</xsl:if>
	<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="para">
	<p xmlns="http://www.w3.org/1999/xhtml">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
        </xsl:if>
	<xsl:apply-templates />
	</p>
</xsl:template>

<xsl:template match="programlisting">
	<pre><xsl:apply-templates /></pre>
</xsl:template>

<xsl:template match="qandaentry">
	<div class="qandaentry"><xsl:apply-templates /></div>
</xsl:template>

<xsl:template match="question">
	<div class="question">
		<span class="questionmark">Question :</span>
		<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="tip">
	<div class="tip">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
	</xsl:if>
	<xsl:apply-templates />
	</div>
</xsl:template>

<xsl:template match="warning">
	<div class="warning">
	<xsl:if test="string-length(@id) > 0">
          <xsl:attribute name="id">
            <xsl:value-of select="@id" />
          </xsl:attribute>
	</xsl:if>
	<xsl:apply-templates />
	</div>
</xsl:template>

</xsl:stylesheet>
