<?xml version="1.0" encoding="ISO-8859-15"?>
<xsl:stylesheet version="1.0"
     xmlns="http://www.w3.org/1999/xhtml"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- 

Achille. An assistant to build static websites compliant to W3C standards.
Copyright (C) 2003 David Soulayrol <dsoulayrol@free.fr>

This file is part of Achille.

Achille is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-->

<!-- http://aspn.activestate.com/ASPN/Cookbook/XSLT/Recipe/65426 -->
<xsl:template name="replace-string">
  <xsl:param name="text" />
  <xsl:param name="from" />
  <xsl:param name="to" />

  <xsl:choose>
    <xsl:when test="contains($text, $from)">

      <xsl:variable name="before" select="substring-before($text, $from)" />
      <xsl:variable name="after" select="substring-after($text, $from)" />

      <xsl:value-of select="$before" />
      <xsl:value-of select="$to" />
      <xsl:call-template name="replace-string">
	<xsl:with-param name="text" select="$after" />
	<xsl:with-param name="from" select="$from" />
	<xsl:with-param name="to" select="$to" />
      </xsl:call-template>
    </xsl:when> 

    <xsl:otherwise>
      <xsl:value-of select="$text" />
    </xsl:otherwise>

  </xsl:choose>            
</xsl:template>

<xsl:template name="basename">
  <xsl:param name="path" />

  <xsl:choose>
    <xsl:when test="contains($path, '/')">
      <xsl:call-template name="basename">
        <xsl:with-param name="path" select="substring-after($path, '/')" />
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$path" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="basename-without-ext">
  <xsl:param name="path" />
  <xsl:variable name="basename">
    <xsl:call-template name="basename">
      <xsl:with-param name="path" select="$path" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="substring-before($basename, '.')" />
</xsl:template>

<xsl:template name="translate-url">
  <xsl:param name="url" />

  <xsl:choose>
    <xsl:when test='starts-with($url, "http:")'>
      <xsl:value-of select="$url" />
    </xsl:when> 

    <xsl:when test='starts-with($url, "/")'>
      <xsl:call-template name="replace-string">
        <xsl:with-param name="text" select="concat($path_to_root, $url)" />
          <xsl:with-param name="from" select="'.xml'" />
          <xsl:with-param name="to" select="'.html'" />
        </xsl:call-template>
    </xsl:when> 
 
    <xsl:otherwise>
      <xsl:call-template name="replace-string">
	    <xsl:with-param name="text" select="$url" />
	    <xsl:with-param name="from" select="'.xml'" />
	    <xsl:with-param name="to" select="'.html'" />
 	  </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>            

</xsl:template>

</xsl:stylesheet>
